@@ -322,8 +322,8 @@ If optional argument P is present test at that point instead of `point'."
       (forward-line -1))
     (goto-char (line-end-position))))
 
-(declare-function outline-next-visible-heading "outline")
 (declare-function outline-on-heading-p "outline")
+(declare-function outline-next-visible-heading "outline")
 
 (beginend-define-mode outline-mode
   (progn
@@ -331,17 +331,50 @@ If optional argument P is present test at that point instead of `point'."
       (outline-next-visible-heading 1)))
   (progn))
 
-(declare-function org-with-limited-levels "org-macs")
-(declare-function outline-next-visible-heading "outline")
 (declare-function org-on-heading-p "org")
 
 (beginend-define-mode org-mode
   (progn
-    (unless (org-on-heading-p)
-      (org-with-limited-levels
-       (outline-next-visible-heading 1))))
+    (while (and (not (org-on-heading-p)) (not (eobp)))
+      (forward-line))
+    (when (eobp)
+      (setf (point) (point-min))))
   (progn))
 
+(beginend-define-mode LaTeX-mode
+  (progn
+    (when (search-forward "\\begin{document}" nil t)
+      (skip-chars-forward "\t\n\r ")))
+  (progn
+    (when (search-backward "\\end{document}" nil t)
+      (skip-chars-backward "\t\n\r "))))
+
+(beginend-define-mode latex-mode
+  (progn
+    (when (search-forward "\\begin{document}" nil t)
+      (skip-chars-forward "\t\n\r ")))
+  (progn
+    (when (search-backward "\\end{document}" nil t)
+      (skip-chars-backward "\t\n\r "))))
+
+(declare-function rg-next-file "rg")
+(declare-function rg-prev-file "rg")
+
+(beginend-define-mode rg-mode
+  (progn
+    (rg-next-file 1))
+  (progn
+    (rg-prev-file 1)))
+
+(beginend-define-mode epa-key-list-mode
+  (progn
+    (re-search-forward "^  [^ ] [^ ]" nil t)
+    (backward-char))
+  (progn
+    (forward-line -1)
+    (re-search-forward "^  [^ ] [^ ]" nil t)
+    (backward-char)))
+
 
 
 ;;;###autoload
@@ -384,4 +417,4 @@ and `compilation-mode' will have their \\[beginning-of-buffer] and
 
 ;;; beginend.el ends here
 
-;;  LocalWords:  beginend
+;;  LocalWords:  beginend whitespace
